#ifndef ANNC
#define ANNC

/* C interfarce for ANN */

#ifdef __cplusplus
extern "C" {
#endif
  /**
     \file ANN_C.h
     \brief Simple C interface.

     Simple C interface for the Yershova and Lavalle KDTree implementation.
     This only covers the basic functionalities.
   */

  /**
     \brief The KDtree handler type.

     This is a dummy type used to hold a reference to the tree.
   */
  typedef void TKDTree;

  /**
     \brief Creates a KDtree (simple version).

     Initializes a KDtree assuming an R topoloty and a uniform
     scale (1) for all dimensions.

     \param dim Dimensionality of the space where the tree is defined.
     
     \return The KDTree handler.
   */
  TKDTree *CreateKDTree(int dim);

  /**
     \brief Creates a KDtree.

     Initializes a KDtree with the given topoloty.

     \param dim Dimensionality of the space where the tree is defined.
     \param topology The topoloty for each dimension (1 Real, 2 angle,
                     3 angle between vectors in 3d).

     \return The KDTree handler.
   */
  TKDTree *CreateKDTreeT(int dim,int *topology);

  /**
     \brief Creates a KDtree.

     Initializes a KDtree with the given topoloty and a
     scale for each dimension.

     \param dim Dimensionality of the space where the tree is defined.
     \param topology The topoloty for each dimension (1 Real, 2 angle,
                     3 angle between vectors in 3d).
     \param scale Scale for each dimension.

     \return The KDTree handler.
   */
  TKDTree *CreateKDTreeTS(int dim,int *topology,double *scale);

  /**
     \brief Adds a point to the tree.

     Adds the given point to the tree.

     \param p The point to add.
     \param tree The handler of the tree where to add the point.
   */
  void AddPoint2KDTree(double *p,TKDTree *tree);

  /**
     \brief KDtree query.

     Determines the nearest neighbour for a given point.

     \param q The query point.
     \param ndx Index for the nearest point in the map.
     \param d The distance to the nearest neighbour.
     \param tree The handler of the tree to query.
   */
  void QueryKDTree(double *q,int *ndx,double *d,TKDTree *tree);

  /**
     \brief KDtree query.

     Determines the neighbours in a given radius for a given point.

     \param q The query point.
     \param r The radius.
     \param k The number of detected neighbours.
     \param ndx Index for the nearest point in the map.
     \param d The distance to the nearest neighbour.
     \param tree The handler of the tree to query.
   */
  void QueryKDTreeR(double *q,double r,int *k,
		    int **ndx,double **d,TKDTree *tree);
  /**
     \brief Destructor.

     KDTree destructor.

     \param tree The tree to delete.
  */
  void DeleteKDTree(TKDTree *tree);


#ifdef __cplusplus
}
#endif

#endif
